#!/usr/bin/perl

use v5.14;
use utf8;
use strict;
use warnings;
use diagnostics;

my $version = '1.00';

use Getopt::Long;
use Cwd 'abs_path';
use File::Basename;
use HTML::Entities qw(decode_entities);

my %mcwsparams = (
    username => '',
    password => '',
);
my $debug = 0;
my %opts;

my @opt_config = (
    'debug|d'        => sub { $debug = 1 },
    'password|P=s'   => sub { ($mcwsparams{'username'}, $mcwsparams{'password'}) = split(/:/, $_[1], 2) },
);

{
local $SIG{__WARN__} = sub { say "\n*** ", $_[0]; };
Getopt::Long::Configure('no_ignore_case');
GetOptions(\%opts, @opt_config) or Usage(1);
}

my $notifier_exe = join '/', dirname(abs_path($0)) , 'DDMountainNotifier-master/MountainNotifier';
my $proxyapp_path = "$ENV{'HOME'}/Library/Application Support/MountainNotifier";

# remove the extant proxy app path which can get left behind
qx/rm -rf "$proxyapp_path"/;

my $secs_between_posts = 2;

sub debug;
my $mcws = MCUtils::MCWS->connect(%mcwsparams);

my $first_time = 1;
my $secs_since_post = 999999;	# never posted
my %info;
while (1) {
    $_ = $mcws->get_playback_info();
    unless (/<Response Status="OK">/) {
	debug 'Unable to obtain MC playback status information: ', $_, ': Sleeping...';
	sleep 5;
	next;
    }

    while (m#<Item Name="([^"]+)">(.+?)</Item>\R#msg) {
	$info{$1} = $2;
    }

    if (!exists $info{'Status'} or $info{'Status'} eq 'Stopped') {
	debug 'Playback Stopped: Sleeping...';
	sleep 3;
	next;
    }

    $info{'Artist'} = decode_entities($info{'Artist'});
    $info{'Album'}  = decode_entities($info{'Album'});
    $info{'Name'}   = decode_entities($info{'Name'});

    my $post_notification = 0;
    if ($first_time) {
	$post_notification = 1;
    }
    elsif ($info{'PositionMS'} <= 5000 and $secs_since_post >= $secs_between_posts) {
	$post_notification = 1;
    }

    if ($post_notification) {
	$first_time = 0;

	# ElapsedTimeDisplay:	0:35
	# RemainingTimeDisplay:	6:58
	# TotalTimeDisplay:	7:33
	# PositionDisplay:	0:35 / 7:33

	debug "Artist: $info{'Artist'}, Album: $info{'Album'}, Track name: $info{'Name'}";

	do_fork_exec($notifier_exe, 'com.jriver.MediaCenter19',
	    "$info{'Name'} [$info{'PlayingNowPositionDisplay'}]",
	    $info{'Artist'},
	    $info{'Album'}
	);

	$secs_since_post = 0;
    }
    printf "Sleeping: %d: Position: %s\n", $secs_since_post++, $info{'RemainingTimeDisplay'} if $debug;
    sleep 1;
}

sub do_fork_exec {
   use English '-no_match_vars';
   my $pid;
   die "Can't fork: $!" unless defined($pid = open(KID, "-|"));
   if ($pid) {           # parent
       while (<KID>) {
	   # never any output
       }
       sleep 5;
       close KID;
   } else {
       my @temp     = ($EUID, $EGID);
       my $orig_uid = $UID;
       my $orig_gid = $GID;
       $EUID = $UID;
       $EGID = $GID;
       # Drop privileges
       $UID  = $orig_uid;
       $GID  = $orig_gid;
       # Make sure privs are really gone
       ($EUID, $EGID) = @temp;
       die "Can't drop privileges"
	   unless $UID == $EUID  && $GID eq $EGID;
       $ENV{PATH} = "/bin:/usr/bin"; # Minimal PATH.
       # Consider sanitizing the environment even more.
    
       exec @_
	   or die "can't exec myprog: $!";
   }
}

sub debug {
    return unless $debug;

    printf "%-20s: %s\n", (split(/::/, (caller(1))[3] || 'main'))[-1], join('', @_);
}

#-----------------------
package MCUtils::MCWS;

use diagnostics;
use v5.14;
use utf8;
use strict;
use warnings;

use URI::Escape;
use LWP::UserAgent;

=pod
    Constructor
	my $mcws = MCWS->connect(%mcwsparams);
=cut

sub connect {
    my $invocant = shift;
    my $class = ref($invocant) || $invocant;
    my $self = {
	 ua		=> LWP::UserAgent->new(),
	 timeout	=> 10,
	 agent		=> 'Mozilla/5.0',
	 server		=> 'localhost:52199',
	 realm		=> 'Library Server',
	 @_,
    };

    $self->{'urlpath'} = '/MCWS/v1';
    $self->{'url'} = 'http://' . $self->{'server'} .  '/MCWS/v1/Playback/Info?Zone=-1';
    $self->{'ua'}->agent($self->{'agent'});
    $self->{'ua'}->timeout(10);
    $self->{'ua'}->env_proxy;
    $self->{'ua'}->cookie_jar({});
    $self->{'ua'}->credentials($self->{'server'}, $self->{'realm'}, $self->{'username'} => $self->{'password'});

    bless($self,$class);
    return $self;
}

sub get_playback_info {
    my $self = shift;

    #http://localhost:52199/MCWS/v1/Playback/Info?Zone=-1
    main::debug "Sending: ", $self->{'url'};

    my $response = $self->{'ua'}->get($self->{'url'});
    unless ($response->is_success) {
	$response->status_line eq '401 Authorization Required' and
	    die "Authenticationo is enabled; you need to use the --password option.\nStopped";

	return $response->status_line;
    }
    $response->content =~ /<\?xml version=.*? \?>.*<Response Status="OK">/ms or
	warn 'Unexpected server response: ', $response->content;

    return $response->content;
}


1;
